###############################################################################
# This perl module contains the configuration information for this parser.
# If you don't understand the format of this configuration file DON'T TOUCH IT.
###############################################################################
# Licensed Materials - Property of IBM
#
# 5724-W86 5724-X63
#
# (C) Copyright IBM Corp. 2003, 2011. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
###############################################################################

package EngineConfig;

@ISA    = qw(Exporter);
@EXPORT = qw(engine_config);

#use diagnostics;
use strict;
use AudLog;
use Exporter();
use Time::Local;

use DDM_MANIP_Config;
use OBJ_ID_XFORM;

my $counter_rename = do "counter_rename.pm";

my %monthHash = (
	'01' => "Jan",
	'02' => "Feb",
	'03' => "Mar",
	'04' => "Apr",
	'05' => "May",
	'06' => "Jun",
	'07' => "Jul",
	'08' => "Aug",
	'09' => "Sep",
	'10' => "Oct",
	'11' => "Nov",
	'12' => "Dec"
);
my %rev_month = reverse(%monthHash);

#-------------------------------------------------------------------------------
# Replace special characters from string and strip leading and trailing blanks
#-------------------------------------------------------------------------------
sub strip_string {
	my ($in_string) = @_;
	$in_string =~ tr/\.\-\ \=\(\)/______/;
	$in_string =~ s/_+/_/g;
	$in_string =~ s/^_//;
	$in_string =~ s/_$//;
	return $in_string;
}

#-------------------------------------------------------------------------------
# Modify/Add rules here
#-------------------------------------------------------------------------------

#my @rules = () ;
my @rules = (
	{
		RULE_TYPE              => 'GENERIC_CSV_GATEWAY',
		RULE_DESC              => 'HUAWEI UTRAN R11 - Config file',
		INPUT_FILE_DESCRIPTION => [
			'Huawei_UTRAN_NodeB_Relationship\.csv',
			'Huawei_UTRAN_NodeB_Cell_relationship\.csv'
		],
		INPUT_DIR_DEPTH        => 4,
		ORDER_OF_FILES         => 'OLDEST_FIRST',
		OVERWRITE_PIF          => 'True',
		FIELD_SEPARATOR        => ',',
		FILENAME_HEADER_FIELDS => {
			#need at least one header field for postparser to work
			FILENAME => '(.*)',			
		},
		DATA_FIELDS                   => [ 0 .. 4 ],
		INVALID_VALUE_MATCH           => [],
		WHITESPACE_MODIFIER           => '_',
		FIELDS_TO_KEY_PIF_FILENAME    => [],
		OUTPUT_BLOCK_NAME             => 'HUAWEI_HIER',
		DEFAULT_NULL_VALUE            => '',
		UPPERCASE_COUNTERS            => 0,
		HEADER_DATA_RECORD_PROCESSING => sub {
			my ( $blk_name_ref, $h_ref, $d_ref ) = @_;

			foreach my $data_key ( keys %{$d_ref} ) {
				$d_ref->{$data_key} =~ s/^NodeB_Id=//g;
				$d_ref->{$data_key} =~ tr/\=/_/;
				$d_ref->{$data_key} =~ s/#N\/A/UNKNOWN/g;
			}

		},
	},

	{
		RULE_TYPE => 'XML_3GPP',
		RULE_DESC => '3gpp xml interface',

		#XML_3GPP_HANDLER_PACKAGE_NAME => 'XML_3GPP_Handler',
		XML_3GPP_HANDLER_PACKAGE_NAME => 'XML_3GPP_Huawei_GPRS_Handler',
		INPUT_FILE_DESCRIPTION        => ['.*\.xml'],
		INPUT_DIR_DEPTH               => 4,
		DIRECTORY_HEADER_FIELDS       => {
			NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
			REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
		},
		ORDER_OF_FILES         => 'OLDEST_FIRST',
		FILENAME_HEADER_FIELDS => {
			FILENAME  => '(.*\.xml)',
			Type      => '^(A|B|C|D).+$',
			StartDate => '^\w{1}(\d{8}).+$',
			StartTime => '^\w{1}\d{8}\.(\d{4}[+-]\d{4}).+$',
			UniqueId  => '^\w{1}\d{8}\.\d{4}.+_(\d+)\.xml$',
		},

		XML_3GPP_FILENAME_VALIDATION => {
			Type      => '^(A|B|C|D).+$',
			StartDate => '^\w{1}(\d{8}).+$',
			StartTime => '^\w{1}\d{8}\.(\d{4}).+$',
		},

		XML_UNIQUE_PIF_BLOCK_HEADER_ELEMENT_NAMES => ['measInfo', 'granPeriod' ],
		XML_HEADER_INFO_FOR_PIF_FILENAME          => ['FILENAME'],
		XML_UNIQUE_PIF_BLOCK_ELEMENT_NAME         => 'measInfo',
		XML_MEASUREMENT_TYPE_ELEMENT_SEQ          => 'measTypes',
		XML_MEASUREMENT_VALUE_ELEMENT_NAME        => 'measValue',
		XML_MEASUREMENT_RESULTS_ELEMENT_SEQ       => 'measResults',
		XML_OUTPUT_BLOCK_ELEMENT_NAME             => 'measInfo',
		XML_OUTPUT_BLOCK_DESCRIPTION              => '^\d{9}$',
		XML_OBJECT_INSTANCE_ID_ELEMENT_NAME       => 'measValue',
		XML_ENABLE_PIF_PER_MOID                   => '0',
		XML_SUSPECT_FLAG_ELEMENT_NAME             => 'suspect',
		XML_PIF_HEADER_ELEMENT_NAMES              => [''],
		XML_LIST_DELIMITER                        => " ",
		XML_MAX_COUNTER_NAME_SIZE                 => 1000,
		XML_DDM_MANIP_RULES                       => ddm_manip_config(),
		HEADER_DATA_RECORD_PROCESSING             => sub {
			my ( $blk_ref, $h_ref, $d_ref ) = @_;
			my ( $cname_new, $tmp_value );
			my $labelcount = 1;

			#-----------------------------------------
			# BLOCKNAME SECTION
			#-----------------------------------------
			# Prevent of 'HEADER' block Processing
			return 0 if ( $$blk_ref =~ /HEADER/ );

			# Rename blockname by prepending 'B'
			if ( $h_ref->{measInfoId} ) {
				$$blk_ref = 'B' . $h_ref->{measInfoId};
			}
			else {
				return 0;
			}

			#-----------------------------------------
			# HEADER SECTION
			#-----------------------------------------
			# Rename and modify date name and format
			if ( exists $h_ref->{StartDate} ) {
				my $date = $h_ref->{StartDate};

				$date =~ /(\d+)(\w{3})(\d{4})/;
				$date = "$3-$rev_month{$2}-$1";
				$d_ref->{START_DATE} = $date;
			}

			# Rename counter and make available at data section
			$d_ref->{GP}         = $h_ref->{duration}  if ( exists $h_ref->{duration} );
			$d_ref->{START_TIME} = $h_ref->{StartTime} if ( exists $h_ref->{StartTime} );

			# Remove redundant header counters
			delete $h_ref->{endTime_GMT_Difference} if exists $h_ref->{endTime_GMT_Difference};
			delete $h_ref->{endTime_Date}           if exists $h_ref->{endTime_Date};
			delete $h_ref->{endTime_Time}           if exists $h_ref->{endTime_Time};

			#-----------------------------------------
			# DATA SECTION
			#-----------------------------------------
			#Rename counter name
			foreach my $cname_ori ( keys %{$d_ref} ) {
				$cname_new = $cname_ori;

				# Replace illegal characters
				$cname_new = &EngineConfig::strip_string($cname_new);

				# Prepend numberic ocunters with C
				if ( $cname_new =~ /^(\d+)$/ ) {
					$cname_new = 'C' . $1;
				}

				# Rename measObjLdn to Object
				elsif ( $cname_new =~ /^measObjLdn$/ ) {
					$cname_new = 'Object_Name';
				}

				# Rename counters
				$tmp_value = $d_ref->{$cname_ori};
				delete $d_ref->{$cname_ori};
				$d_ref->{$cname_new} = $tmp_value;
			}

			foreach my $cvalue ( values %$d_ref ) {
				#$cvalue = $1 if ( $cvalue =~ /(.+)e\+0\d+/ );    #comment for alm00200438
				#-----------------------------------------------------------------------
	            # To convert exponential value into int8   #add for alm00200438
	            #------------------------------------------------------------------------
				my $origval = $cvalue;
				if($cvalue =~ /^(\d*\.?\d*)(?:e|E)(-|\+)?(\d+)$/) {
					my $newval = $cvalue * 1;
					if ($newval eq "0") {
						$cvalue = $origval;
					} else {
						$cvalue = $newval;
					}
				}
				$cvalue = '' if ( $cvalue =~ /^NIL$/ );
				$cvalue =~ s/\ /\_/g;
			}

			if ( exists( $d_ref->{Object_Name} ) ) {
				$d_ref->{Object_Name} =~ s/([^\w])\ (\w+)/\1\2/g;
				OBJ_ID_XFORM::process_object_name( $d_ref, $$blk_ref, 'Object_Name' );
			}

			# Quote the value for specific counters
			#if ( exists $d_ref->{Object_Name} ) {
			#    my $cvalue = $d_ref->{Object_Name};
			#    $d_ref->{Object_Name} = "\"$cvalue\"";
			#}

			if ( ( $$blk_ref eq 'B67109416' ) || ( $$blk_ref eq 'B67109417' ) ) {
				$d_ref->{SS7_LINKSET_ID} = $d_ref->{OBJECT_ID};

				my $obj_id = $d_ref->{OBJECT_ID};
				$d_ref->{SS7_POINT_ID} = $d_ref->{RNC_ID};

				if ( $d_ref->{SS7_LINKSET_ID} =~ /MTP3BLinkSet/ ) {
					$d_ref->{SS7_LINKSET_ID} =~ s/\/MTP3BLink\/.*//g;
					if ( !( $d_ref->{SS7_LINKSET_ID} =~ /^RNC/ ) ) {
						$d_ref->{SS7_LINKSET_ID} = $d_ref->{RNC_ID} . "/" . $d_ref->{SS7_LINKSET_ID};
					}
				}
				else {
					$d_ref->{SS7_LINKSET_ID} = $1 if $d_ref->{SS7_LINKSET_ID} =~ /\/(MTP3BLink.*)/;
					$d_ref->{SS7_LINKSET_ID} =~ s/\/SLC_.*//g;
					if ( !( $d_ref->{SS7_LINKSET_ID} =~ /^RNC/ ) ) {
						$d_ref->{SS7_LINKSET_ID} = $d_ref->{RNC_ID} . "/" . $d_ref->{SS7_LINKSET_ID};
					}
				}
			}

			if ( $$blk_ref eq 'B67109453' ) {
				if ( ( $d_ref->{PROC1_ID} eq 'SPU' ) || ( $d_ref->{PROC1_ID} eq 'CPUS' ) ) {    #v900r011, CPUS added
					                                                                            #$$blk_name_ref .= '_V200';
					$d_ref->{Processor_Type} = $d_ref->{PROC1_ID};
				}
				else {
					$d_ref->{Processor_Type} = '';
				}
			}

			if ( $$blk_ref eq 'B67109549' ) {
				if ( $d_ref->{CELL_ID} eq '' ) {
					if ( $d_ref->{OBJECT_ID} =~ /^UOI/ ) {
						$d_ref->{Proc_Type} = "UOI";
					}
					elsif ( $d_ref->{OBJECT_ID} =~ /^FIBER/ ) {

						#v900r011
						$d_ref->{Proc_Type} = 'FIBER';
					}
				}
				else {
					$d_ref->{Proc_Type} = "CELL";
				}
			}

			#------------------------------------------------------------------
			# R11 to filter RNC from OBJECT_ID
			# eg Before OBJECT_ID is RNC401/GCU/SRN_0/SN_12/BT_GCU/SSN_0
			# After OBJECT_ID is GCU/SRN_0/SN_12/BT_GCU/SSN_0
			#------------------------------------------------------------------
			my ( $f1, $f2, $f3 ) = split( '/', $d_ref->{OBJECT_ID}, 3 );
			if ( $f1 =~ /^RNC/ ) {
				$f2 .= '/';
				$f2 .= $f3;
				$d_ref->{OBJECT_ID} = $f2;
			}

			#commented for alm00155654-------------
			# ## convert RNC_ID to RNC_Id (v900r011)
			# if (exists $d_ref->{RNC_ID})
			# {
				# $d_ref->{RNC_Id} = $d_ref->{RNC_ID};
			# }

			# ## convert CELL_ID to Cell_Id (v900r011)
			# if (exists $d_ref->{CELL_ID})
			# {
				# $d_ref->{Cell_Id} = $d_ref->{CELL_ID};
			# }
			#--------------------------------------

			## workaround for missing Cell_Id and Cell_Label (v900r011)
			if ($d_ref->{Cell_Id} eq '' || $d_ref->{Cell_Label} eq '')
			{
				if (exists $d_ref->{OBJECT_ID} && $d_ref->{OBJECT_ID} =~ /.*\/(Label.*)\/(CellID.*)/)
				{
					$d_ref->{Cell_Id} = $2;
					$d_ref->{CELL_ID} = $2;
					$d_ref->{Cell_Label} = $1;
				}
				if (exists $d_ref->{NEIGHBOUR_ID} && $d_ref->{NEIGHBOUR_ID} =~ /.*\/(Label.*)\/(CellID.*)/)
				{
					$d_ref->{Cell_Id} = $2;
					$d_ref->{CELL_ID} = $2;
					$d_ref->{Cell_Label} = $1;
				}
			}

			## workaround for missing MTP3LinkSet_Id (v900r011)
			if ($d_ref->{MTP3LinkSet_Id} eq '')
			{
				if (exists $d_ref->{OBJECT_ID} && $d_ref->{OBJECT_ID} =~ /(MTP3.*\/.*)\/.*/)
				{
					$d_ref->{MTP3LinkSet_Id} = $1;
				}
			}

			return 0;
		}
	},

	{
		RULE_TYPE              => 'GENERIC_CSV_GATEWAY',
		RULE_DESC              => 'HUAWEI UTRAN R6/R7/R8/R10 - Data files',
		INPUT_FILE_DESCRIPTION => ['pmresult_.*_\d{12}_\d{12}.*\.csv'],
		INPUT_DIR_DEPTH        => 4,
		PRE_PARSE              => 'HUAWEI_UTRAN',
		ORDER_OF_FILES         => 'OLDEST_FIRST',

		FIELD_SEPARATOR            => ',',
		HEADER_FIELDS              => [],
		DATA_FIELDS                => [ 0 .. 200 ],
		INVALID_VALUE_MATCH        => [],
		WHITESPACE_MODIFIER        => '_',
		FIELDS_TO_KEY_PIF_FILENAME => [],
		OUTPUT_BLOCK_NAME          => 'HUAWEI_UTRAN',
		DEFAULT_NULL_VALUE         => '',
		UPPERCASE_COUNTERS         => 0,

		#HEADERLINES_TO_SKIP     => 2,
		FILENAME_HEADER_FIELDS => {
			FILENAME  => '(.*)',
			BLOCKNAME => 'pmresult_(\d+)_\d+_\d{12}_\d{12}.*\.csv',	
		},
		DIRECTORY_HEADER_FIELDS => {
			NETWORK_ID => '\/NETWORK_?([^\/]+)\/?',
			REGION_ID  => '\/[^\/]+\/REGION_?([^\/]+)\/?',
		},
		QUICK_DATA_SPLIT => 'True',

		HEADER_DATA_RECORD_PROCESSING => sub {
			my ( $blk_name_ref, $h_ref, $d_ref ) = @_;

			# Set block name
			$$blk_name_ref = "B" . $h_ref->{BLOCKNAME};

			# Set date and time
			#if ($d_ref->{Result_Time} =~ /(\d{4}-\d{2}-\d{2})_(\d{2}:\d{2})/) {
			if ( $d_ref->{Result_Time} =~ /(\d{4}-\d{1,2}-\d{1,2})_(\d{1,2}:\d{1,2})/ ) {
				$d_ref->{START_DATE} = $1;
				$d_ref->{START_TIME} = $2;
				$d_ref->{GP}         = $d_ref->{Granularity_Period} * 60;
				delete $d_ref->{Result_Time};
				delete $d_ref->{Granularity_Period};
			}

			#added for alm00200438
			foreach my $cvalue ( values %$d_ref ) {
				#-----------------------------------------------------------------------
				# To convert exponential value into int8
				#------------------------------------------------------------------------
				my $origval = $cvalue;
				if($cvalue =~ /^(\d*\.?\d*)(?:e|E)(-|\+)?(\d+)$/) {
					my $newval = $cvalue * 1;
					if ($newval eq "0") {
						$cvalue = $origval;
					} else {
						$cvalue = $newval;
					}
				}
			}
			
			# process object names
			if ( exists( $d_ref->{Object_Name} ) ) {
				OBJ_ID_XFORM::process_object_name( $d_ref, $$blk_name_ref, 'Object_Name' );
			}

			if ( $$blk_name_ref eq 'B67109416' ) {
				$d_ref->{SS7_LINKSET_ID} = $d_ref->{OBJECT_ID};

				my $obj_id = $d_ref->{OBJECT_ID};
				$d_ref->{SS7_POINT_ID} = $d_ref->{RNC_ID};
				if ( $d_ref->{SS7_LINKSET_ID} =~ /MTP3BLinkSet/ ) {
					$d_ref->{SS7_LINKSET_ID} =~ s/\/MTP3BLink\/.*//g;
					if ( !( $d_ref->{SS7_LINKSET_ID} =~ /^RNC/ ) ) {
						$d_ref->{SS7_LINKSET_ID} = $d_ref->{RNC_ID} . "/" . $d_ref->{SS7_LINKSET_ID};
					}
				}
				else {
					$d_ref->{SS7_LINKSET_ID} = $1 if $d_ref->{SS7_LINKSET_ID} =~ /\/(MTP3BLink.*)/;
					$d_ref->{SS7_LINKSET_ID} =~ s/\/SLC_.*//g;
					if ( !( $d_ref->{SS7_LINKSET_ID} =~ /^RNC/ ) ) {
						$d_ref->{SS7_LINKSET_ID} = $d_ref->{RNC_ID} . "/" . $d_ref->{SS7_LINKSET_ID};
					}
				}
			}

			if ( $$blk_name_ref eq 'B67109453' || ( $d_ref->{PROC1_ID} eq 'CPUS' ) ) {    #v900r011, CPUS added
				if ( $d_ref->{PROC1_ID} eq 'SPU' ) {

					#$$blk_name_ref .= '_V200';
					$d_ref->{Processor_Type} = $d_ref->{PROC1_ID};
				}
				else {
					$d_ref->{Processor_Type} = '';
				}
			}

			#R11
			if ( $$blk_name_ref eq 'B67109549' ) {
				if ( $d_ref->{CELL_ID} eq '' ) {
					$d_ref->{Proc_Type} = "UOI";
				}
				elsif ( $d_ref->{OBJECT_ID} =~ /^FIBER/ ) {

					#v900r011
					$d_ref->{Proc_Type} = 'FIBER';
				}
				else {
					$d_ref->{Proc_Type} = "CELL";
				}
			}

			#do counter rename
			foreach my $cname ( keys %$d_ref ) {
				my $cname_new = $counter_rename->{ $$blk_name_ref . '.' . lc($cname) };
				if ( defined $cname_new ) {
					$d_ref->{$cname_new} = $d_ref->{$cname};
					delete $d_ref->{$cname};
				}
			}

			#--------------------------------------------------------------
			# R11 to filter RNC from OBJECT_ID
			# eg Before OBJECT_ID is RNC401/GCU/SRN_0/SN_12/BT_GCU/SSN_0
			# After OBJECT_ID is GCU/SRN_0/SN_12/BT_GCU/SSN_0
			#--------------------------------------------------------------
			my ( $f1, $f2, $f3 ) = split( '/', $d_ref->{OBJECT_ID}, 3 );
			if ( $f1 =~ /^RNC/ ) {
				$f2 .= '/';
				$f2 .= $f3;
				$d_ref->{OBJECT_ID} = $f2;
			}

			#commented for alm00155654-------------
			# ## convert RNC_ID to RNC_Id
			# if (exists $d_ref->{RNC_ID})
			# {
				# $d_ref->{RNC_Id} = $d_ref->{RNC_ID};
			# }

			# ## convert CELL_ID to Cell_Id
			# if (exists $d_ref->{CELL_ID})
			# {
				# $d_ref->{Cell_Id} = $d_ref->{CELL_ID};
			# }
			#--------------------------------------

			# workaround for missing Cell_Id and Cell_Label (alm00225570)
            # for Object ID which has ?/BSC6900UCell:Label=?, CellID=?
            if ($d_ref->{CELL_ID} eq '' || $d_ref->{Cell_Label} eq '')
            {
	            if (exists $d_ref->{OBJECT_ID} && $d_ref->{OBJECT_ID} =~ /.*\/(Label.*)\/(CellID.*)/)
	            {
		            $d_ref->{CELL_ID} = $2;
		            $d_ref->{Cell_Label} = $1;
	            }
            }

			return 0;
		},

	},

	{
		RULE_TYPE => 'GENERIC_CSV_GATEWAY',
		RULE_DESC => 'Network and Region Configuration file',
		#INPUT_FILE_DESCRIPTION => ['Network_Region_RNC\.csv'],
		INPUT_FILE_DESCRIPTION     => ['Huawei_UTRAN_Network_Region\.csv'],
		INPUT_DIR_DEPTH            => 4,
		ORDER_OF_FILES             => 'OLDEST_FIRST',
		OVERWRITE_PIF          => 'True',    #add for alm00198609
		FIELD_SEPARATOR            => ',',
		HEADER_FIELDS              => [0],
		DATA_FIELDS                => [ 0 .. 3 ],
		INVALID_VALUE_MATCH        => [],
		WHITESPACE_MODIFIER        => '_',
		FIELDS_TO_KEY_PIF_FILENAME => [],
		OUTPUT_BLOCK_NAME          => 'Network_Region',
		DEFAULT_NULL_VALUE         => '0',
		UPPERCASE_COUNTERS         => 0,
		HEADERLINES_TO_SKIP        => 0,
		FILENAME_HEADER_FIELDS     => {
			FILENAME => '(.*)',
		},
	}

);

sub engine_config {
	return \@rules;
}

1;


